@@ -31,6 +31,7 @@ from typing import Any
 from typing import Dict
 from typing import List
 from typing import Optional
+from typing import Tuple
 
 from openshift.dynamic.exceptions import NotFoundError as OpenShiftNotFoundError
 from .exceptions import ThothCommonException
@@ -41,9 +42,10 @@ from .exceptions import NotFoundException
 from .exceptions import ConfigurationError
 from .exceptions import SolverNameParseError
 from .helpers import (
-    get_service_account_token,
-    _get_incluster_token_file,
     _get_incluster_ca_file,
+    _get_incluster_token_file,
+    get_service_account_token,
+    normalize_os_version,
 )
 from .enums import ThothAdviserIntegrationEnum
 
@@ -187,17 +189,6 @@ class OpenShift:
             self._workflow_manager = WorkflowManager(openshift=self)
         return self._workflow_manager
 
-    @staticmethod
-    def normalize_os_version(
-        os_name: Optional[str], os_version: Optional[str]
-    ) -> Optional[str]:
-        """Normalize operating system version based on operating system used."""
-        if os_name is None or os_version is None or os_name.lower() != "rhel":
-            return os_version
-
-        # Discard any minor release, if present.
-        return os_version.split(".", maxsplit=1)[0]
-
     @classmethod
     def parse_python_solver_name(cls, solver_name: str) -> Dict[str, Any]:
         """Parse os and Python identifiers encoded into solver name."""
@@ -226,7 +217,7 @@ class OpenShift:
         python_version = ".".join(list(python_version))
         return {
             "os_name": parts[0],
-            "os_version": cls.normalize_os_version(parts[0], parts[1]),
+            "os_version": normalize_os_version(parts[0], parts[1]),
             "python_version": python_version,
         }
 
@@ -242,7 +233,7 @@ class OpenShift:
             return solver
 
         os_name = runtime_environment["operating_system"].get("name")
-        os_version = cls.normalize_os_version(
+        os_version = normalize_os_version(
             operating_system.get("name"), operating_system.get("version")
         )
         python_version = runtime_environment.get("python_version")
@@ -332,7 +323,9 @@ class OpenShift:
 
         # TODO: rewrite to OpenShift rest client once it will support it.
         endpoint = "{}/api/v1/namespaces/{}/pods/{}/log".format(
-            self.openshift_api_url, namespace, pod_id,
+            self.openshift_api_url,
+            namespace,
+            pod_id,
         )
 
         response = requests.get(
@@ -377,21 +370,35 @@ class OpenShift:
 
         return response.text
 
-    def get_workflow_node_log(
+    def get_workflow_pod_name(
         self, node_name: str, workflow_id: str, namespace: str
-    ) -> Optional[str]:
-        """Get log from a task/node in a workflow."""
+    ) -> str:
+        """Get pod name where task is being executed."""
         workflow = self.get_workflow(workflow_id, namespace=namespace)
         nodes = workflow.get("status", {}).get("nodes", {})
 
         for pod_name, node_info in nodes.items():
             if node_info["displayName"] == node_name:
-                return self.get_pod_log(pod_name, namespace=namespace, container="main")
+                return pod_name  # type: ignore
 
         raise NotFoundException(
-            f"No node {node_name!r} in workflow {workflow_id!r} in namespace {namespace!r} found"
+            f"No node named {node_name!r} found in workflow {workflow_id!r} in namespace {namespace!r}"
         )
 
+    def get_workflow_node_log(
+        self, node_name: str, workflow_id: str, namespace: str
+    ) -> Optional[str]:
+        """Get log from a task/node in a workflow."""
+        pod_name = self.get_workflow_pod_name(node_name, workflow_id, namespace)
+        return self.get_pod_log(pod_name, namespace=namespace, container="main")
+
+    def get_workflow_node_status(
+        self, node_name: str, workflow_id: str, namespace: str
+    ) -> Dict[str, Any]:
+        """Get status from a task/node in a workflow."""
+        pod_id = self.get_workflow_pod_name(node_name, workflow_id, namespace)
+        return self.get_pod_status(pod_id, namespace)
+
     def get_build(self, build_id: str, namespace: str) -> Dict[str, Any]:
         """Get a build in the given namespace."""
         # TODO: rewrite to OpenShift rest client once it will support it.
@@ -1179,7 +1186,9 @@ class OpenShift:
             )
 
     @staticmethod
-    def verify_kebechet_inputs(origin: Optional[str],) -> None:
+    def verify_kebechet_inputs(
+        origin: Optional[str],
+    ) -> None:
         """Verify if Thoth Kebechet integration inputs are correct."""
         parameters = locals()
         if not all(parameters.values()):
@@ -1395,14 +1404,24 @@ class OpenShift:
         )
 
     def schedule_mi_workflow(
-        self, repository: str, *, job_id: Optional[str] = None,
+        self,
+        repository: str,
+        entities: Optional[str] = None,
+        *,
+        job_id: Optional[str] = None,
     ) -> Optional[str]:
         """Schedule Meta-information Indicators Workflow.
 
         :param repository:str: GitHub repository in full name format: <repo_owner>/<repo_name>
+        :param entities:Optional[str]: Meta-information Indicator Entities that will be inspected
+                                       multiple entities are in form of Foo,Bar,...
         """
         workflow_id = job_id or self.generate_id("mi")
-        template_parameters = {"WORKFLOW_ID": workflow_id, "REPOSITORY": repository}
+        template_parameters = {
+            "WORKFLOW_ID": workflow_id,
+            "REPOSITORY": repository,
+            "ENTITIES": entities,
+        }
 
         return self._schedule_workflow(
             workflow=self.workflow_manager.submit_mi,
@@ -1413,7 +1432,10 @@ class OpenShift:
         )
 
     def schedule_kebechet_workflow(
-        self, webhook_payload: Dict[str, Any], *, job_id: Optional[str] = None,
+        self,
+        webhook_payload: Dict[str, Any],
+        *,
+        job_id: Optional[str] = None,
     ) -> Optional[str]:
         """Schedule Kebechet Workflow for a Webhook from GitHub App.."""
         workflow_id = job_id or self.generate_id("kebechet-job")
@@ -1430,6 +1452,57 @@ class OpenShift:
             },
         )
 
+    def schedule_kebechet_administrator(
+        self,
+        message_info: Dict[str, str],
+        message_type: str,
+        *,
+        job_id: Optional[str] = None,
+    ) -> Optional[str]:
+        """Schedule Kebechet Administrator Workflow for an internal trigger message."""
+        workflow_id = job_id or self.generate_id("kebechet-administrator")
+        template_parameters = {
+            "WORKFLOW_ID": workflow_id,
+            "THOTH_PACKAGE_NAME": message_info.get("PACKAGE_NAME"),
+            "THOTH_PACKAGE_VERSION": message_info.get(
+                "PACKAGE_VERSION"
+            ),  # Optional value
+            "THOTH_PACKAGE_INDEX": message_info.get("PACKAGE_INDEX"),
+            "THOTH_SOLVER_NAME": message_info.get("SOLVER_NAME"),  # Optional value
+            "THOTH_MESSAGE_TYPE": message_type,
+        }
+
+        return self._schedule_workflow(
+            workflow=self.workflow_manager.submit_kebechet_administrator,
+            parameters={
+                "template_parameters": template_parameters,
+                "workflow_parameters": {},
+            },
+        )
+
+    def schedule_kebechet_run_url_workflow(
+        self,
+        repo_url: str,
+        service_name: str = "github",
+        *,
+        job_id: Optional[str] = None,
+    ) -> Optional[str]:
+        """Schedule Kebechet Workflow for a particular valid slug and service name."""
+        workflow_id = job_id or self.generate_id("kebechet-run-url")
+        template_parameters = {
+            "WORKFLOW_ID": workflow_id,
+            "KEBECHET_REPO_URL": repo_url,
+            "KEBECHET_SERVICE_NAME": service_name,
+        }
+
+        return self._schedule_workflow(
+            workflow=self.workflow_manager.submit_kebechet_run_url,
+            parameters={
+                "template_parameters": template_parameters,
+                "workflow_parameters": {},
+            },
+        )
+
     def schedule_security_indicator(
         self,
         python_package_name: str,
@@ -1463,6 +1536,35 @@ class OpenShift:
             },
         )
 
+    def schedule_graph_sync(
+        self,
+        document_id: str,
+        force_sync: bool = False,
+        *,
+        job_id: Optional[str] = None,
+    ) -> Optional[str]:
+        """Schedule graph sync for specific document id."""
+        if not self.middletier_namespace:
+            raise ConfigurationError(
+                "Unable to schedule graph-sync without middletier namespace being set"
+            )
+
+        graph_sync_id = job_id or self.generate_id("graph-sync")
+        template_parameters = {
+            "THOTH_SYNC_JOB_ID": graph_sync_id,
+            "THOTH_SYNC_DOCUMENT_ID": document_id,
+            "THOTH_FORCE_SYNC": force_sync,
+        }
+        workflow_parameters = self._assign_workflow_parameters_for_ceph()
+
+        return self._schedule_workflow(
+            workflow=self.workflow_manager.submit_graph_sync,
+            parameters={
+                "template_parameters": template_parameters,
+                "workflow_parameters": workflow_parameters,
+            },
+        )
+
     def _raise_on_invalid_response_size(
         self,
         response: Any,
@@ -1486,8 +1588,10 @@ class OpenShift:
         # TODO: This does not work - see issue reported upstream:
         #   https://github.com/openshift/openshift-restclient-python/issues/190
         # return TemplateOpenshiftIoApi().create_namespaced_processed_template_v1(namespace, template)
-        endpoint = "{}/apis/template.openshift.io/v1/namespaces/{}/processedtemplates".format(
-            self.openshift_api_url, namespace
+        endpoint = (
+            "{}/apis/template.openshift.io/v1/namespaces/{}/processedtemplates".format(
+                self.openshift_api_url, namespace
+            )
         )
         response = requests.post(
             endpoint,
@@ -1513,6 +1617,43 @@ class OpenShift:
         result: Dict[str, Any] = response.json()
         return result
 
+    def get_mi_repositories_and_organizations(
+        self,
+    ) -> Tuple[List[str], List[str]]:
+        """Get all of the repositories and organizations for mi-analysis.
+
+        :rtype: Tuple of (repositories, organizations)
+        """
+        if not self.infra_namespace:
+            raise ConfigurationError(
+                "Infra namespace is required in order to get repositories"
+            )
+
+        cm = self.get_configmap(
+            configmap_id="mi-scheduler", namespace=self.infra_namespace
+        )
+
+        organizations = cm["data"].get("organizations", "")
+        _LOGGER.info(
+            "Detected %s organizations from configMap for inspection", organizations
+        )
+        repositories = cm["data"].get("repositories", "")
+        _LOGGER.info(
+            "Detected %s repositories from configMap for inspection", repositories
+        )
+
+        orgs = (
+            organizations.split(",")
+            if organizations is not None and organizations != ""
+            else []
+        )
+        repos = (
+            repositories.split(",")
+            if repositories is not None and repositories != ""
+            else []
+        )
+        return (repos, orgs)
+
     @staticmethod
     def parse_cpu_spec(cpu_spec: typing.Optional[str]) -> typing.Optional[float]:
         """Parse the given CPU requirement as used by OpenShift/Kubernetes."""
